/*#######################################################################################
AVR Small Webserver

Copyright (C) 2004 Ulrich Radig

Bei Fragen und Verbesserungen wendet euch per EMail an

mail@ulrichradig.de

oder im Forum meiner Web Page : www.ulrichradig.de


Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation verffentlicht,
weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
(nach Ihrer Option) jeder spteren Version.

Die Verffentlichung dieses Programms erfolgt in der Hoffnung,
da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT
FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
Programm erhalten haben.
Falls nicht, schreiben Sie an die Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#######################################################################################*/

#include "main.h"
#include "compatible.h"
#include <util/delay.h>
#include <compat/deprecated.h>
#include "arp.h"
#include "interrupt4.h"
#include "rtl8019.h"
#include "tcp.h"
#include "ip.h"
#include "icmp.h"
#include "clock.h"
#include "uart.h"
#include "ftpd.h"
#include "webpage.h"
#include "telnet.h"

#include "board.h"
#include "ADC.h"
#include "SHTxx.h"
#include "tcp_client.h"

const unsigned char mymac[] = {30,10,43,43,45,77};	///< MAC number assigned to board
const unsigned char MYIP[] = {192,168,2,10};		///< IP address assigned to board
const unsigned char GateWay[] = {192,168,2,1};		///< gateway IP
uint8_t u8_netmask = 24;							///< netmask's bits

extern char szTelnetBuffer[35];
unsigned long ul_PseudoRandom = 0;



unsigned char reset_flags;
   
//############################################################################
//Hauptprogramm
int main (void)
//############################################################################
{
    /* Read and clear reset flags */
    reset_flags = MCUSR;                // Save reset flags.
    MCUSR       = 0;                    // Clear all flags previously set.

    init_beeper();
    init_LED1();
    init_LED2();
    Led1On();
    Led2On();
    beep();
    Led1Off();
    Led2Off();
    UART_Init();	//see uart.h for baudrate!

    printf ("** Isa Webserver **\n");
	printf ("MCUCSR: %d\r", reset_flags);

    //Grundinizialisierung der Netzwerkkarte
    Init_Realtek_Network_Card();
    if (ReadRTL(0) == 34) printf ("OK");
    else
    {
        beep();
        beep();
        beep();
        printf ("Init: ERROR(%d)",ReadRTL(0));
    }

    //Ausgabe meiner MAC Adresse auf Serieller Schnittstelle
    printf ("\nMy Mac: %x.%x.%x.%x.%x.%x\n",mymac[0],mymac[1],mymac[2]
            ,mymac[3],mymac[4],mymac[5]);

    //Ausgabe meiner IP Adresse auf Serieller Schnittstelle
    printf ("My IP : %d.%d.%d.%d\n\n",MYIP[0],MYIP[1],MYIP[2],MYIP[3]);

    //Globale Interrupts Einschalten
    sei ();

    //Interrupt 4 Mega103/128 Interrupt 0 bei Mega32
    //soll auslsen bei ansteigender Flanke
    Networkcard_INT_Set();

    //Interrupt 4 oder 0 Einschalten (fr die Netzwerkkarte)
    Networkcard_INT_Enable();

#if USE_CLOCK
    Start_Clock();
#endif

    //Hauptprogramm luft stndig in einer schleife und macht nichts
    uint8_t error = 0;
    value humid_val,temp_val;
    char buf[20], bufPressure[10];

	ADC_init();
	#define ADC_BUF_SIZE 10
	uint16_t ADCbuf[ADC_BUF_SIZE];
	uint8_t ADCptr = 0;
	for (uint8_t i=0; i<ADC_BUF_SIZE; i++)
		ADCbuf[i] = ADC_read();
	uint32_t ADCsum;
	uint16_t ADCavg;

    while (1)
    {
        for (uint8_t temp=0;temp<200;temp++)
        {
            if (temp%4==0)
                Led1Toggle();
            _delay_loop_2(0xFFFF);
        }
		
		ADCbuf[ADCptr++] = ADC_read();
		ADCptr = ADCptr % ADC_BUF_SIZE;
		ADCsum = 0;
		for (uint8_t i=0; i<ADC_BUF_SIZE; i++)
			ADCsum += ADCbuf[i];
		ADCavg = ADCsum/ADC_BUF_SIZE;
		//printf("Pressure: %d ADC\r\n", ADCavg);
		float fPressure;
		fPressure = 0.0887 * ADCavg + 970.3;	//coefficient from LMS method calibration, individual calibration needed!
		itoa((int)(fPressure + 0.5), bufPressure, 10);		
		
		
        s_connectionreset();
        error = 0;
        unsigned char checksum;
        //WARNING! s_measure may take up to 2s (max timeout), so you'll probably
        //have to rewrite this function into non-blocking code if your application does sth time-critical
        error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP);
        error+=s_measure((unsigned char*) &humid_val.i,&checksum,HUMID);
        if (error != 0)
        {
            s_connectionreset();
            cli();
            strcpy(szTelnetBuffer, "MEAS ERROR\r\n");
            sei();
        }
        else
        {
            humid_val.f=(float)humid_val.i;
            temp_val.f=(float)temp_val.i;
            calc_sht11(&humid_val.f,&temp_val.f);
            temp_val.i=(int32_t)(temp_val.f*10);

            cli();
            strcpy(szTelnetBuffer, "Temp: ");
            if (temp_val.i < 0)
                strcat(szTelnetBuffer, "-");
            temp_val.i = abs(temp_val.i);
            itoa(temp_val.i/10, buf, 10);
            strcat(szTelnetBuffer, buf);
            strcat(szTelnetBuffer,".");
            itoa(temp_val.i%10, buf, 10);
            strcat(szTelnetBuffer,buf);
            strcat(szTelnetBuffer,"C  ");

            humid_val.i = (int32_t)(humid_val.f);
            strcat(szTelnetBuffer,"RH: ");
            itoa(humid_val.i, buf, 10);
            strcat(szTelnetBuffer,buf);
            strcat(szTelnetBuffer,"%  ");
			strcat(szTelnetBuffer,bufPressure);
			strcat(szTelnetBuffer,"hPa");
            sei();
        }
		ul_PseudoRandom--;
#if USE_TCP_CLIENT
		//client and server uses the same buffer!
		cli();
		TcpClientConnectProcess();
		sei();
#endif
    }
    return (1);
}

